SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
IF OBJECT_ID(N'dbo.usp_LoadRule') IS NOT NULL
    DROP PROCEDURE dbo.usp_LoadRule;
GO
-- =============================================
-- Author:		Aaron N. Cutshall
-- Create date: 29-Jul-2017
-- Description:	Load measure rule into database tables
-- =============================================
CREATE PROCEDURE [dbo].[usp_LoadRule]
	@xmlSpec XML, @ParentRuleId INT, @RuleId INT OUTPUT
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @iSeq INT,
			@nSeq INT,
			@FactId INT,
			@SubRuleId INT,
			@xmlFrag XML;

	DECLARE @OutputTbl TABLE (Id INT);

	-- Load Rule
	INSERT INTO dbo.tbl_Rule (
			RuleName,
			ParentRuleId,
			LogicalId
	        )
	OUTPUT Inserted.RuleId INTO @OutputTbl(Id)
	SELECT	a.value('(@name)', 'varchar(64)'),
			@ParentRuleId,
			CASE a.value('(@logical)', 'varchar(3)') WHEN 'and' THEN 1 WHEN 'or' THEN 2 ELSE 3 END
	FROM	@xmlSpec.nodes('*') x(a);

	SELECT	@RuleId = Id FROM @OutputTbl;
	DELETE FROM @OutputTbl;

	-- Load assertions
	INSERT INTO dbo.tbl_RuleFact (
			RuleId,
			FactId
			)
	SELECT	@RuleId,
			b.FactId
	FROM	@xmlSpec.nodes('*/fact') x(a)
	INNER JOIN dbo.tbl_Fact b
		ON b.FactName = a.value('(@name)', 'varchar(64)');

	-- Load rules
	SELECT @iSeq = 0, @nSeq = ISNULL(COUNT(*),0)
	FROM @xmlSpec.nodes('*/rule') x(a);
	
	WHILE @iSeq < @nSeq BEGIN
		SET @iSeq = @iSeq + 1;
		SET @xmlFrag = @xmlSpec.query('*/rule[position()=sql:variable("@iSeq")]');
		EXEC dbo.usp_LoadRule @xmlFrag, @RuleId, @SubRuleId OUTPUT;
	END
END
